<?php
/**
* @Package Tunnel to CPG
* @copyright (c) 2020 RJCreations
* @license http://opensource.org/licenses/GPL-3.0 GNU Public License Version 3
*/

namespace Drupal\cpgtunnel\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

class CPGtunnelForm extends ConfigFormBase
{

	public function getFormId ()
	{
		return 'cpgtunnel_form';
	}


	public function buildForm (array $form, FormStateInterface $form_state)
	{
		$config = $this->config('cpgtunnel.settings');

		$form['roles']['roles'] = [
			'#type' => 'checkboxes',
			'#size' => 3,
			'#options' => array_map('\Drupal\Component\Utility\Html::escape', user_role_names(TRUE)),
			'#multiple' => TRUE,
			'#title' => $this->t('Roles that will tunnel to CPG'),
			'#default_value' => $config->get('groups'),
			'#description' => $this->t('Only the selected Roles will have a tunnel to CPG created when they log in..')
		];

		$form['secret'] = [
			'#type' => 'textfield',
			'#title' => $this->t('Encryption phrase'),
			'#size' => 60,
			'#maxlength' => 128,
			'#description' => $this->t('Enter a tunnel encryption phrase (same as set at CPG)'),
			'#required' => TRUE,
			'#default_value' => $config->get('secret')
		];

		$form['actions']['#type'] = 'actions';
		$form['actions']['submit'] = [
			'#type' => 'submit',
			'#value' => $this->t('Save'),
			'#button_type' => 'primary'
		];

		return $form;
	}


	public function submitForm (array &$form, FormStateInterface $form_state)
	{
		$this->config('cpgtunnel.settings')
			->set('secret', $form_state->getValue('secret'))
			->set('groups', array_filter($form_state->getValue('roles')))
			->save();

		parent::submitForm($form, $form_state);
	}


	protected function getEditableConfigNames() {
		return ['cpgtunnel.settings'];
	}

}
